==> bnctool-0.0.10.tgz: package/bin/bnc-build.js <==
#!/usr/bin/env node

var program = require('commander');
var chalk = require('chalk');
var path = require('path');
var util = require('util');
var UglifyJS = require("uglify-js");
var file = require('../lib/file');
var logger = require('../lib/logger');
var TPL = require('../lib/template');

program
    .option('-w, --watch', 'monitor the changes of project, auto build')
    .option('-u, --uglify', 'uglify all files')
    .parse(process.argv);

/**
 * Read all files in directory and find config.json
 */
var base = process.cwd();
var fileList = file.scanDirectory(base);
var configPath = arraySearch(fileList, 'config.json');
if(!configPath) {
    logger.log(chalk.red('cannot find config.json!'));
    return false;
}

/**
 * Get config and location work directory
 * for watching/compiling
 */
var config = file.read(configPath);
logger.log('config:');
logger.log(chalk.blue(config));
var workDir = path.dirname(configPath);
logger.log();
logger.log('work directory ' + workDir + ':');
var workList = file.scanDirectory(workDir);
logger.log(workList.join('\n'));

/**
 * Pack all css files and html files to js file
 */
var bncStatic = {
    html: {},
    css: {}
};
var bncJS = [];
var timer = null;
var template = '(function(){\nvar bncStatic=%s;\n%s})();\n';
var content;
var outputDir = path.join(base, 'output');
var output = path.join(outputDir, 'entry.js');

if(config) {
    config = JSON.parse(config);
    file.mkdir(path.join(base, 'output'));
    file.mkdir(path.join(base, 'output', 'images'));

    var html = config.html;
    var css = config.css;
    var js = config.js;

    if(html && html.length > 0) {
        html.map(function (value) {
            var tmp = path.join(workDir, value);
            bncStatic.html[value] = file.read(tmp);
        });
    }
    if(css && css.length > 0) {
        css.map(function (value) {
            var tmp = path.join(workDir, value);
            bncStatic.css[value] = file.read(tmp);
        });
    }
    bncJS = js.map(function (value) {
        var tmp = path.join(workDir, value);
        return file.read(tmp);
    });

    copyImage();

    makeHTML();

    build();
}

if(program.watch) {
    file.watchDir(workDir, function (type, filename) {
        logger.log('%s..........%s',type, filename);
        filename = path.join(workDir, filename);
        if(filename === configPath) {
            if(type === 'change') {
                config = file.read(configPath);
            } else {
                logger.fatal('cannot find config.json. please check it once!');
                process.abort();
            }
        } else {
            var ext = path.extname(filename);
            switch (ext) {
                case '.js':
                    compileJS(bncJS, config);
                    break;
                case '.css':
                    compileCSS(bncStatic, type, filename);
                    break;
                case '.html':
                    compileHTML(bncStatic, type, filename);
                    break;
                default:
                    copyImage();
            }
        }

        if(timer) {
            clearTimeout(timer);
        }
        timer = setTimeout(function () {
            build();
        }, 500);
    })
} else {
    logger.log(chalk.green('bnc build complete!'));
}

function copyImage() {
    var imgList = arraySearchAll(workList, 'images');
    imgList.map(function (value) {
        var name = path.basename(value);
        file.copy(value, path.join(base, 'output', 'images', name));
    });
}

function makeHTML() {
    var writeContent = TPL.html_pc.replace('{{title}}', config.id);
    var cmd = util.format('BNC.ability.%s.%s()', config.id, config.initFunction);
    writeContent = writeContent.replace('{{caller}}', cmd);

    file.write(path.join(outputDir, 'index.html'), writeContent);
}

function compileJS(bncJS, config) {
    bncJS = config.js.map(function (value) {
        var tmp = path.join(workDir, value);
        return file.read(tmp);
    });
}

function compileCSS(bncStatic, type, filename) {
    var value = path.basename(filename);
    bncStatic.css[value] = file.read(filename);
}

function compileHTML(bncStatic, type, filename) {
    var value = path.basename(filename);
    bncStatic.html[value] = file.read(filename);
}

function build() {
    var jsCode = bncJS.join(';').replace(/BNC\.use\(/, 'BNC.use(bncStatic,');
    if(program.uglify) {
        content = util.format(template, JSON.stringify(bncStatic), UglifyJS.minify(jsCode).code);
    } else {
        content = util.format(template, JSON.stringify(bncStatic), jsCode);
    }
    logger.log(chalk.green('release: ' + output));
    file.write(output, content);
    content = '';
}

function arraySearch(array, keyword) {
    for (var i = 0; i < array.length; i++) {
        var item = array[i];
        var reg = new RegExp(keyword, 'ig');
        if(reg.test(item)) {
            return item;
        }
    }
    return '';
}

function arraySearchAll(array, keyword) {
    var ret = [];
    array.forEach(function (value) {
        var reg = new RegExp(keyword, 'ig');
        if(reg.test(value)) {
            ret.push(value);
        }
    });
    return ret;
}

==> bnctool-0.1.1.tgz: package/bin/bnc-build.js <==
#!/usr/bin/env node

var program = require('commander');
var chalk = require('chalk');
var path = require('path');
var util = require('util');
var UglifyJS = require("uglify-js");
var file = require('../lib/file');
var logger = require('../lib/logger');

program
    .option('-w, --watch', 'monitor the changes of project, auto build')
    .option('-u, --uglify', 'uglify all files')
    .parse(process.argv);

/**
 * Read all files in directory and find config.json
 */
var base = process.cwd();
var fileList = file.scanDirectory(base);
var configPath = arraySearch(fileList, 'config.json');
if(!configPath) {
    logger.log(chalk.red('cannot find config.json!'));
    return false;
}

/**
 * Get config and location work directory
 * for watching/compiling
 */
var config = file.read(configPath);
logger.log('config:');
console.log(chalk.blue(config));
var workDir = path.dirname(configPath);
logger.log();
logger.log('work directory ' + workDir);
var workList = file.scanDirectory(workDir);
console.log(workList.join('\n'));

/**
 * Pack all css files and html files to js file
 */
var bncStatic = {
    html: {},
    css: {}
};
var bncJS = [];
var timer = null;
var template = '(function(){\nvar bncStatic=%s;\n%s})();\n';
var content;
var outputDir = path.join(base, 'debug');
var output = path.join(outputDir, 'entry.js');

if(config) {
    config = JSON.parse(config);
    file.mkdir(path.join(base, 'debug'));
    file.mkdir(path.join(base, 'debug', 'images'));

    var html = config.html;
    var css = config.css;
    var js = config.js;

    if(html && html.length > 0) {
        html.map(function (value) {
            var tmp = path.join(workDir, value);
            bncStatic.html[value] = file.read(tmp);
        });
    }
    if(css && css.length > 0) {
        css.map(function (value) {
            var tmp = path.join(workDir, value);
            bncStatic.css[value] = file.read(tmp);
        });
    }
    bncJS = js.map(function (value) {
        var tmp = path.join(workDir, value);
        return file.read(tmp);
    });

    copyImage();

    build();
}

if(program.watch) {
    file.watchDir(workDir, function (type, filename) {
        logger.log('%s..........%s',type, filename);
        filename = path.join(workDir, filename);
        if(filename === configPath) {
            if(type === 'change') {
                config = file.read(configPath);
            } else {
                logger.fatal('cannot find config.json. please check it once!');
                process.abort();
            }
        } else {
            var ext = path.extname(filename);
            switch (ext) {
                case '.js':
                    compileJS(config);
                    break;
                case '.css':
                    compileCSS(bncStatic, type, filename);
                    break;
                case '.html':
                    compileHTML(bncStatic, type, filename);
                    break;
                default:
                    copyImage();
            }
        }

        if(timer) {
            clearTimeout(timer);
        }
        timer = setTimeout(function () {
            build();
        }, 500);
    })
} else {
    logger.log(chalk.green('bnc build complete!'));
}

function copyImage() {
    var imgList = arraySearchAll(workList, 'images');
    imgList.map(function (value) {
        var name = path.basename(value);
        file.copy(value, path.join(base, 'debug', 'images', name));
    });
}

function compileJS(config) {
    bncJS = config.js.map(function (value) {
        var tmp = path.join(workDir, value);
        return file.read(tmp);
    });
}

function compileCSS(bncStatic, type, filename) {
    var value = path.basename(filename);
    bncStatic.css[value] = file.read(filename);
}

function compileHTML(bncStatic, type, filename) {
    var value = path.basename(filename);
    bncStatic.html[value] = file.read(filename);
}

function build() {
    var jsCode = bncJS.join(';').replace(/BNC\.use\(/, 'BNC.use(bncStatic,');
    if(program.uglify) {
        content = util.format(template, JSON.stringify(bncStatic), UglifyJS.minify(jsCode).code);
    } else {
        content = util.format(template, JSON.stringify(bncStatic), jsCode);
    }
    logger.log(chalk.green('release: ' + output));
    file.write(output, content);
    content = '';
}

function arraySearch(array, keyword) {
    for (var i = 0; i < array.length; i++) {
        var item = array[i];
        var reg = new RegExp(keyword, 'ig');
        if(reg.test(item)) {
            return item;
        }
    }
    return '';
}

function arraySearchAll(array, keyword) {
    var ret = [];
    array.forEach(function (value) {
        var reg = new RegExp(keyword, 'ig');
        if(reg.test(value)) {
            ret.push(value);
        }
    });
    return ret;
}
